Attribute VB_Name = "modFunktionen"
Option Explicit
'die folgenden Funktionen sind Beigabe und werden im Buch nicht besprochen
'die Bezeichnungen entsprechen den in Excel integrierten Funktionen zur Zinseszinsrechnung

'benutzerdefinierte Funktionen zur einfachen Zinsrechnung

Public Function ZWEV(ZINSEV, ZZREV, RMZEV, Optional BWEV, Optional F) As Double
    If IsMissing(BWEV) Then BWEV = 0
    If IsMissing(F) Then F = 0
    ZWEV = -BWEV * (1 + ZINSEV * ZZREV) - ZZREV * RMZEV * (1 + ZINSEV * ((ZZREV - 1) / 2 + F))
End Function

Public Function RMZEV(ZINSEV, ZZREV, BWEV, Optional ZWEV, Optional F) As Double
    If IsMissing(ZWEV) Then ZWEV = 0
    If IsMissing(F) Then F = 0
    RMZEV = -ZWEV - BWEV * (1 + ZZREV * ZINSEV)
    RMZEV = RMZEV / ZZREV
    RMZEV = RMZEV / (1 + (ZZREV - 1) / 2 * ZINSEV + F * ZINSEV)
End Function

Public Function BWEV(ZINSEV, ZZREV, RMZEV, Optional ZWEV, Optional F) As Double
    If IsMissing(ZWEV) Then ZWEV = 0
    If IsMissing(F) Then F = 0
    BWEV = (-ZWEV - ZZREV * RMZEV * (1 + (ZZREV - 1) / 2 * ZINSEV + F * ZINSEV)) / (1 + ZINSEV * ZZREV)
End Function

Public Function ZINSEV(ZZREV, RMZEV, BWEV, Optional ZWEV, Optional F) As Double
    If IsMissing(ZWEV) Then ZWEV = 0
    If IsMissing(F) Then F = 0
    ZINSEV = (-ZWEV - BWEV - ZZREV * RMZEV) / (ZZREV * BWEV + ZZREV * RMZEV * ((ZZREV - 1) / 2 + F))
End Function

Public Function ZZREV(ZINSEV, RMZEV, BWEV, Optional ZWEV, Optional F) As Double
    Dim p As Double
    Dim q As Double
    Dim w As Double
    
    If IsMissing(ZWEV) Then ZWEV = 0
    If IsMissing(F) Then F = 0
    If RMZEV = 0 Then
       ZZREV = (-ZWEV / BWEV - 1) / ZINSEV
    Else
       p = 2 * (RMZEV * (1 + (F - 1 / 2) * ZINSEV) + BWEV * ZINSEV) / (RMZEV * ZINSEV)
       q = 2 * (ZWEV + BWEV) / (RMZEV * ZINSEV)
       w = Sqr(p * p / 4 - q)
      ZZREV = -p / 2 + w
    End If
End Function

Public Function KUMKAPITALR(ZZR, BW, ZRA, ZRE) As Double
    KUMKAPITALR = -BW * (ZRE - ZRA + 1) / ZZR
End Function

Public Function KAPZR(ZZR, BW) As Double
    KAPZR = -BW / ZZR
End Function

Public Function ZINSZR(ZINS, ZR, ZZR, BW) As Double
    ZINSZR = -BW * (1 - (ZR - 1) / ZZR) * ZINS
End Function

Public Function BWW(DS, LZ, WS) As Double
    BWW = -WS * (1 - DS * LZ / 360)
End Function

Public Function BWD(ZINS, DYN, ZZR, ANF) As Double
    Dim faktor As Double
    If ZINS = DYN Then
        BWD = -ZZR * ANF
    Else
        faktor = (1 + DYN) / (1 + ZINS)
        BWD = -ANF * (faktor ^ ZZR - 1) / (faktor - 1)
    End If
End Function

Public Function XKAPITALWERT360(ZINS, WERTE, ZEITPKTE) As Double
    Dim summe As Double
    Dim i As Integer
    Dim letzter As Double
    
    Application.Volatile
    summe = 0
    i = 1
    Do While Not IsEmpty(WERTE(i))
        letzter = ZEITPKTE(i)
        i = i + 1
    Loop
    i = 1
    Do While Not IsEmpty(WERTE(i))
        summe = summe + WERTE(i) * (1 + ZINS * (Application.Days360(ZEITPKTE(i), letzter, True) / 360))
        i = i + 1
    Loop
    XKAPITALWERT360 = summe / (1 + ZINS * (Application.Days360(ZEITPKTE(1), letzter, True) / 360))
End Function

